/*
 * Project: MoleCuilder
 * Description: creates and alters molecular systems
 * Copyright (C)  2010-2012 University of Bonn. All rights reserved.
 * 
 *
 *   This file is part of MoleCuilder.
 *
 *    MoleCuilder is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    MoleCuilder is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with MoleCuilder.  If not, see <http://www.gnu.org/licenses/>.
 */

/*
 * SetRandomNumbersEngine.cpp
 *
 *  Created on: Jan 01, 2011
 *      Author: heber
 */

// include config.h
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "CodePatterns/MemDebug.hpp"

#include "CodePatterns/Log.hpp"
#include "molecule.hpp"
#include "CodePatterns/Verbose.hpp"
#include "World.hpp"

#include "RandomNumbers/RandomNumberEngine.hpp"
#include "RandomNumbers/RandomNumberEngine_Parameters.hpp"
#include "RandomNumbers/RandomNumberEngineFactory.hpp"

#include <iostream>
#include <sstream>
#include <string>

#include "SetRandomNumbersEngineAction.hpp"

using namespace MoleCuilder;

// and construct the stuff
#include "SetRandomNumbersEngineAction.def"
#include "Action_impl_pre.hpp"
/** =========== define the function ====================== */
ActionState::ptr CommandSetRandomNumbersEngineAction::performCall() {
  // note down old type
  std::string oldtype = RandomNumberEngineFactory::getInstance().getCurrentTypeName();
  LOG(1, "Current engine is " << oldtype);

  // set the new default
  RandomNumberEngineFactory::getInstance().setCurrentType(params.engine_type.get());
  LOG(0, "STATUS: Engine of random number generator is now: "
      << RandomNumberEngineFactory::getInstance().getCurrentTypeName());

  // note down old parameters
  RandomNumberEngine_Parameters *oldparameters =
      RandomNumberEngineFactory::getInstance().getPrototype().getParameterSet();
  // set each parameter (that is not -1);
  {
    std::stringstream input(params.parameters.get());
    RandomNumberEngine_Parameters *currentparameters =
        RandomNumberEngineFactory::getInstance().getPrototype().getParameterSet();
    input >> *currentparameters;  // add new values on top
    if (!currentparameters->isDefault()) {
      LOG(1, "Changing prototype's parameters.");
      RandomNumberEngineFactory::getInstance().manipulatePrototype(*currentparameters);
    }
    delete currentparameters;
  }

  {
    RandomNumberEngine_Parameters *currentparameters =
        RandomNumberEngineFactory::getInstance().getPrototype().getParameterSet();
    LOG(1, "Its parameters are: " << *currentparameters);
    delete  currentparameters;
  }

  std::stringstream output;
  output << *oldparameters;
  CommandSetRandomNumbersEngineState *newstate =
      new CommandSetRandomNumbersEngineState(oldtype,output.str(),params);
  delete oldparameters;
  return ActionState::ptr(newstate);
}

ActionState::ptr CommandSetRandomNumbersEngineAction::performUndo(ActionState::ptr _state) {
  CommandSetRandomNumbersEngineState *state =
      assert_cast<CommandSetRandomNumbersEngineState*>(_state.get());

  // note down old type for redo
  std::string newtype = RandomNumberEngineFactory::getInstance().getCurrentTypeName();
  LOG(1, "Newly set engine was " << newtype);

  // note down old parameters for redo
  RandomNumberEngine_Parameters *newparameters =
      RandomNumberEngineFactory::getInstance().getPrototype().getParameterSet();

  // set the new default
  RandomNumberEngineFactory::getInstance().setCurrentType(state->old_engine_type);
  LOG(0, "STATUS: Engine of random number generator is again: "
      << RandomNumberEngineFactory::getInstance().getCurrentTypeName());

  // set each parameter back (that is not -1);
  {
    std::stringstream input(state->old_parameters);
    RandomNumberEngine_Parameters *currentparameters =
        RandomNumberEngineFactory::getInstance().getPrototype().getParameterSet();
    input >> *currentparameters;
    if (!currentparameters->isDefault()) {
      LOG(1, "Changing prototype's parameters.");
      RandomNumberEngineFactory::getInstance().manipulatePrototype(*currentparameters);
    }
    delete currentparameters;
  }

  {
    RandomNumberEngine_Parameters *currentparameters =
        RandomNumberEngineFactory::getInstance().getPrototype().getParameterSet();
    LOG(1, "Its parameters are: " << *currentparameters);
    delete  currentparameters;
  }

  std::stringstream output;
  output << *newparameters;
  CommandSetRandomNumbersEngineState *newstate =
      new CommandSetRandomNumbersEngineState(newtype,output.str(),params);
  delete newparameters;
  return ActionState::ptr(newstate);
}

ActionState::ptr CommandSetRandomNumbersEngineAction::performRedo(ActionState::ptr _state){
  return performUndo(_state);
}

bool CommandSetRandomNumbersEngineAction::canUndo() {
  return true;
}

bool CommandSetRandomNumbersEngineAction::shouldUndo() {
  return true;
}
/** =========== end of function ====================== */
